import Vue, { Component } from 'vue';
import { HeadClient } from '@vueuse/head';
import Router from 'vue-router';
import { Store } from 'vuex';

/**
 * Context that used for render server entry point for development
 * It needs a production implementation for your environment.
 * The context will go to the created hook parameters of the plugin
 */
interface Context {
    hostname: string;
    protocol: string;
    url: string;
    cookies: Record<string, any>;
    ip: string;
    memcache: number | null;
    statusCode: number;
    headers: Record<string, any>;
    responseHeaders: Record<string, any>;
}
type HookResponse = void | {
    head?: HeadClient;
    router?: Router;
    store?: Store<any>;
    inserts?: {
        htmlAttrs?: string;
        headTags?: string;
        bodyAttrs?: string;
        body?: string;
        dependencies?: Array<string>;
    };
    context?: Record<string, any>;
    app?: Vue;
};
type Hook = (params: {
    app?: Vue;
    url: URL | Location;
    isClient: boolean;
    initialState: Record<string, any>;
    context?: Context;
    [key: string]: any;
}) => HookResponse | Promise<HookResponse>;
interface CreatorOptions {
    created?: Hook;
    mounted?: Hook;
    rendered?: Hook;
    serializer?: (state: any) => any | Promise<any>;
    shouldPreload?: (file: string, type: string) => boolean;
    shouldPrefetch?: (file: string, type: string) => boolean;
    mount?: {
        rootContainer?: any;
        hydrating?: boolean;
    };
    rootProps?: Record<string, any> | null;
}
type SsrRenderer = (url: string | URL, options?: {
    manifest?: Record<string, string[]>;
    [key: string]: any;
}) => Promise<{
    html: string;
    dependencies: string[];
}>;
type ClientHandler = (App: Component, options?: CreatorOptions) => Promise<void>;
type SsrHandler = (App: Component, options?: CreatorOptions) => SsrRenderer;

export { CreatorOptions as C, SsrHandler as S, ClientHandler as a, Context as b };
